//
//  CorelPS2PDF.cpp : Defines the entry point for the console application.
//
//  Copyright (C) 2009 Corel Corporation
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2 as
//  published by the Free Software Foundation. See the file License.txt
//  for more details.
//


#include "stdafx.h"

#define PS2PDF_SUCCESS 1
#define PS2PDF_ERROR   1


// function declarations.
bool IsGhostcriptAvailable( CString &strApplicationName );
bool GetGhostscriptExecutable( const CString &strGsDll, CString &strApplicationName );


int _tmain( int argc, _TCHAR* argv[] )
{
	if ( 3 != argc )
	{
		_tprintf_s( _T("usage: CorelPS2PDF input.ps [output.pdf]\n") );
		return PS2PDF_ERROR;
	}

	CString strApplicationName;
	if ( !IsGhostcriptAvailable( strApplicationName ) )
	{
		return PS2PDF_ERROR;
	}

	// set up the command line.
	CString strCommandLine = _T("-q -dSAFER -dNOPAUSE -dBATCH -dEPSCrop -sDEVICE=pdfwrite -dCompatibilityLevel=1.4 -dAutoFilterColorImages=false -dAutoFilterGrayImages=false -dColorImageFilter=/FlateEncode -dGrayImageFilter=/FlateEncode -sOutputFile=");
	// output file.
	strCommandLine += _T("\"");
	strCommandLine += argv[2];
	strCommandLine += _T("\" \"");
	// input file.
	strCommandLine += argv[1];
	strCommandLine += _T("\"");

	STARTUPINFO si;
	::ZeroMemory( &si, sizeof( STARTUPINFO ) );
	si.cb = sizeof( STARTUPINFO );

	PROCESS_INFORMATION pi;
	::ZeroMemory( &pi, sizeof( PROCESS_INFORMATION ) );

	// execute the process.
	INT nRet = ::CreateProcess( strApplicationName, const_cast< LPTSTR >( ( LPCTSTR )strCommandLine ), NULL, NULL, FALSE, CREATE_NO_WINDOW | NORMAL_PRIORITY_CLASS, NULL, NULL, &si, &pi );
	if ( 0 != nRet )
	{
		WaitForSingleObject( pi.hProcess, INFINITE );

		CloseHandle( pi.hProcess );
		CloseHandle( pi.hThread );
	}

	return ( nRet > 0 ? PS2PDF_SUCCESS : PS2PDF_ERROR );
}


//
// IsGhostcriptAvailable looks up the Registry to find out if the
// Ghostscript is installed. If it is not, the function returns
// false. If it is installed, it will parse the registry keys to
// find out the location where it is installed, setting the return
// value to true, and also returning the pathname to the Ghostscript
// executable.
//
bool IsGhostcriptAvailable( CString &strApplicationName )
{
	bool bRet = false;

	HKEY hKeyGhostscript = NULL;

	if ( ERROR_SUCCESS == ::RegOpenKeyEx( HKEY_LOCAL_MACHINE, _T("Software\\GPL Ghostscript"), 0, KEY_READ, &hKeyGhostscript ) )
	{
		// see if we have a preferred version we should use.
		double dPreferredVersion = 0.0;

		HKEY hKeyPreferred = NULL;
		if ( ERROR_SUCCESS == ::RegOpenKeyEx( HKEY_CURRENT_USER, _T("Software\\Corel\\CorelPS2PDF"), 0, KEY_READ, &hKeyPreferred ) )
		{
			// get the value.
			CString strVersion;

			LPTSTR szVersion = strVersion.GetBuffer( 10 );
			DWORD dwVersionSize = 10 * sizeof( _TCHAR );

			DWORD dwType = 0;

			LONG lResult = ::RegQueryValueEx( hKeyPreferred, _T("PreferredVersion"), NULL, &dwType, ( LPBYTE )szVersion, &dwVersionSize );
			strVersion.ReleaseBuffer();

			if ( ( ERROR_SUCCESS == lResult ) && ( REG_SZ == dwType ) )
			{
				dPreferredVersion = _tstof( strVersion );
			}

			::RegCloseKey( hKeyPreferred );
		}

		// enumerate the version keys.
		_TCHAR szVersion[ _MAX_PATH ] = {};
		DWORD dwVersionSize = _MAX_PATH;

		CString strGsDll;

		int nVersionIndex = -1;
		double dVersion = 0.0;

		for ( int i = 0; ; i++ )
		{
			dwVersionSize = _MAX_PATH;

			LONG lResult = ::RegEnumKeyEx( hKeyGhostscript, i, szVersion, &dwVersionSize, NULL, NULL, NULL, NULL );
			if ( ERROR_SUCCESS == lResult )
			{
				// szVersion should have the version number of the Ghostscript installation.
				// open the subkey.
				HKEY hKeyVersion = NULL;

				if ( ERROR_SUCCESS == ::RegOpenKeyEx( hKeyGhostscript, szVersion, 0, KEY_READ, &hKeyVersion ) )
				{
					double dKeyVersion = _tstof( szVersion );

					if ( ( dPreferredVersion > 0 ) && ( dPreferredVersion == dKeyVersion ) )
					{
						nVersionIndex = i;
						break;
					}
					else if ( dKeyVersion > dVersion )
					{
						nVersionIndex = i;
					}

					::RegCloseKey( hKeyVersion );
				}
			}
			else
			{
				break;
			}
		}

		// now get the Ghotscript executable
		if ( nVersionIndex >= 0 )
		{
			dwVersionSize = _MAX_PATH;

			if ( ERROR_SUCCESS == ::RegEnumKeyEx( hKeyGhostscript, nVersionIndex, szVersion, &dwVersionSize, NULL, NULL, NULL, NULL ) )
			{
				HKEY hKeyVersion = NULL;

				if ( ERROR_SUCCESS == ::RegOpenKeyEx( hKeyGhostscript, szVersion, 0, KEY_READ, &hKeyVersion ) )
				{
					// one of the subkeys should be GS_DLL, we want to get its value
					LPTSTR szGsDll = strGsDll.GetBuffer( _MAX_PATH );
					DWORD dwGsDllSize = _MAX_PATH * sizeof( _TCHAR );

					::RegQueryValueEx( hKeyVersion, _T("GS_DLL"), NULL, NULL, ( LPBYTE )szGsDll, &dwGsDllSize );

					strGsDll.ReleaseBuffer();
				}

				::RegCloseKey( hKeyVersion );
			}
		}

		if ( !strGsDll.IsEmpty() )
		{
			bRet = GetGhostscriptExecutable( strGsDll, strApplicationName );
		}

		::RegCloseKey( hKeyGhostscript );
	}

	return bRet;
}


//
// GetGhostscriptExecutable breaks up the pathname of the Ghostscript DLL
// to obrtain the name of the Ghostscript executable. If successfull
// it returns true, and false otherwise.
//
bool GetGhostscriptExecutable( const CString &strGsDll, CString &strApplicationName )
{
	bool bRet = false;

	_TCHAR szGsBinDrive[ _MAX_PATH ] = {};
	_TCHAR szGsBinDir[ _MAX_PATH ] = {};

	// split off the file name.
	if ( 0 == _tsplitpath_s( strGsDll, szGsBinDrive, _MAX_PATH, szGsBinDir, _MAX_PATH, NULL, 0, NULL, 0 ) )
	{
		strApplicationName  = szGsBinDrive;
		strApplicationName += szGsBinDir;
		strApplicationName += _T("gswin32c.exe");

		bRet = true;
	}

	return bRet;
}
